@@ -255,7 +255,7 @@ namespace libtorrent
 #ifndef WIN32_LEAN_AND_MEAN
 #define WIN32_LEAN_AND_MEAN
 #endif
-#include <Windows.h>
+#include <windows.h>
 #include "libtorrent/assert.hpp"
 
 namespace libtorrent
