@@ -65,7 +65,7 @@ namespace libtorrent
 		session_impl& ses
 		, boost::weak_ptr<torrent> tor
 		, shared_ptr<socket_type> s
-		, tcp::endpoint const& remote
+		, tcp::endpoint const& endp
 		, policy::peer* peerinfo)
 		:
 #ifndef NDEBUG
@@ -85,7 +85,7 @@ namespace libtorrent
 		, m_last_receive(time_now())
 		, m_last_sent(time_now())
 		, m_socket(s)
-		, m_remote(remote)
+		, m_remote(endp)
 		, m_torrent(tor)
 		, m_active(true)
 		, m_peer_interested(false)
@@ -145,6 +145,7 @@ namespace libtorrent
 	peer_connection::peer_connection(
 		session_impl& ses
 		, boost::shared_ptr<socket_type> s
+		, tcp::endpoint const& endp
 		, policy::peer* peerinfo)
 		:
 #ifndef NDEBUG
@@ -164,6 +165,7 @@ namespace libtorrent
 		, m_last_receive(time_now())
 		, m_last_sent(time_now())
 		, m_socket(s)
+		, m_remote(endp)
 		, m_active(false)
 		, m_peer_interested(false)
 		, m_peer_choked(true)
@@ -209,8 +211,9 @@ namespace libtorrent
 		m_remote = m_socket->remote_endpoint();
 
 #ifdef TORRENT_VERBOSE_LOGGING
-		TORRENT_ASSERT(m_socket->remote_endpoint() == remote());
-		m_logger = m_ses.create_log(remote().address().to_string() + "_"
+		error_code ec;
+		TORRENT_ASSERT(m_socket->remote_endpoint(ec) == remote());
+		m_logger = m_ses.create_log(remote().address().to_string(ec) + "_"
 			+ boost::lexical_cast<std::string>(remote().port()), m_ses.listen_port());
 		(*m_logger) << "*** INCOMING CONNECTION\n";
 #endif
@@ -1434,10 +1437,8 @@ namespace libtorrent
 		
 			if (t->alerts().should_post(alert::fatal))
 			{
-				if (j.str != "write failed: No space left on device"){
-					std::string err = "torrent paused: disk write error, " + j.str;
-					t->alerts().post_alert(file_error_alert(t->get_handle(), err));
-				}
+				std::string err = "torrent paused: disk write error, " + j.str;
+				t->alerts().post_alert(file_error_alert(t->get_handle(), err));
 			}
 			t->pause();
 			return;
@@ -2022,6 +2023,9 @@ namespace libtorrent
 
 		if (t)
 		{
+			// make sure we keep all the stats!
+			t->add_stats(statistics());
+
 			if (t->has_picker())
 			{
 				piece_picker& picker = t->picker();
