@@ -94,9 +94,12 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <sys/mount.h>
 #endif
 
-#if defined(_WIN32) && defined(UNICODE)
+#if TORRENT_USE_WPATH
 
+#ifdef BOOST_WINDOWS
 #include <windows.h>
+#endif
+
 #include <boost/filesystem/exception.hpp>
 #include "libtorrent/utf8.hpp"
 #include "libtorrent/buffer.hpp"
@@ -249,6 +252,53 @@ namespace
 namespace libtorrent
 {
 
+	template <class Path>
+	void recursive_copy(Path const& old_path, Path const& new_path, std::string& error)
+	{
+		using boost::filesystem::basic_directory_iterator;
+#ifndef BOOST_NO_EXCEPTIONS
+		try {
+#endif
+		TORRENT_ASSERT(error.empty());
+		if (is_directory(old_path))
+		{
+			create_directory(new_path);
+			for (basic_directory_iterator<Path> i(old_path), end; i != end; ++i)
+			{
+				recursive_copy(i->path(), new_path / i->leaf(), error);
+				if (!error.empty()) return;
+			}
+		}
+		else
+		{
+			copy_file(old_path, new_path);
+		}
+#ifndef BOOST_NO_EXCEPTIONS
+		} catch (std::exception& e) { error = e.what(); }
+#endif
+	}
+
+	template <class Path>
+	void recursive_remove(Path const& old_path)
+	{
+		using boost::filesystem::basic_directory_iterator;
+#ifndef BOOST_NO_EXCEPTIONS
+		try {
+#endif
+		if (is_directory(old_path))
+		{
+			for (basic_directory_iterator<Path> i(old_path), end; i != end; ++i)
+				recursive_remove(i->path());
+			remove(old_path);
+		}
+		else
+		{
+			remove(old_path);
+		}
+#ifndef BOOST_NO_EXCEPTIONS
+		} catch (std::exception& e) {}
+#endif
+	}
 	std::vector<std::pair<size_type, std::time_t> > get_filesizes(
 		torrent_info const& t, fs::path p)
 	{
@@ -265,7 +315,7 @@ namespace libtorrent
 				fs::path f = p / i->path;
 				size = file_size_win(f);
 				time = last_write_time_win(f);
-#elif defined(_WIN32) && defined(UNICODE)
+#elif TORRENT_USE_WPATH
 				fs::wpath f = safe_convert((p / i->path).string());
 				size = file_size(f);
 				time = last_write_time(f);
@@ -314,7 +364,7 @@ namespace libtorrent
 				fs::path f = p / i->path;
 				size = file_size_win(f);
 				time = last_write_time_win(f);
-#elif defined(_WIN32) && defined(UNICODE)
+#elif TORRENT_USE_WPATH
 				fs::wpath f = safe_convert((p / i->path).string());
 				size = file_size(f);
 				time = last_write_time(f);
@@ -436,7 +486,7 @@ namespace libtorrent
 				last_path = dir;
 				if (!exists_win(last_path))
 					create_directories_win(last_path);
-#elif defined(_WIN32) && defined(UNICODE)
+#elif TORRENT_USE_WPATH
 				last_path = dir;
 				fs::wpath wp = safe_convert(last_path.string());
 				if (!exists(wp))
@@ -499,8 +549,15 @@ namespace libtorrent
 				std::pair<iter_t, bool> ret = directories.insert((m_save_path / bp).string());
 				bp = bp.branch_path();
 			}
+#if TORRENT_USE_WPATH
+			try
+			{ fs::remove(safe_convert(p)); }
+			catch (std::exception& e)
+			{ error = e.what(); }
+#else
 			if (std::remove(p.c_str()) != 0 && errno != ENOENT)
 				error = std::strerror(errno);
+#endif
 		}
 
 		// remove the directories. Reverse order to delete
@@ -509,8 +566,15 @@ namespace libtorrent
 		for (std::set<std::string>::reverse_iterator i = directories.rbegin()
 			, end(directories.rend()); i != end; ++i)
 		{
+#if TORRENT_USE_WPATH
+			try
+			{ fs::remove(safe_convert(*i)); }
+			catch (std::exception& e)
+			{ error = e.what(); }
+#else
 			if (std::remove(i->c_str()) != 0 && errno != ENOENT)
 				error = std::strerror(errno);
+#endif
 		}
 
 		if (!error.empty()) throw std::runtime_error(error);
@@ -600,7 +664,7 @@ namespace libtorrent
 	// returns true on success
 	bool storage::move_storage(fs::path save_path)
 	{
-#if defined(_WIN32) && defined(UNICODE) && BOOST_VERSION >= 103400
+#if TORRENT_USE_WPATH
 		fs::wpath old_path;
 		fs::wpath new_path;
 #else
@@ -616,7 +680,7 @@ namespace libtorrent
 			CreateDirectory(wsave_path.c_str(), 0);
 		else if ((GetFileAttributes(wsave_path.c_str()) & FILE_ATTRIBUTE_DIRECTORY) == 0)
 			return false;
-#elif defined(_WIN32) && defined(UNICODE)
+#elif TORRENT_USE_WPATH
 		fs::wpath wp = safe_convert(save_path.string());
 		if (!exists(wp))
 			create_directory(wp);
@@ -631,7 +695,7 @@ namespace libtorrent
 
 		m_files.release(this);
 
-#if defined(_WIN32) && defined(UNICODE) && BOOST_VERSION >= 103400
+#if TORRENT_USE_WPATH
 		old_path = safe_convert((m_save_path / m_info->name()).string());
 		new_path = safe_convert((save_path / m_info->name()).string());
 #else
@@ -650,8 +714,17 @@ namespace libtorrent
 			m_save_path = save_path;
 			return true;
 		}
-		catch (std::exception&) {}
-		return false;
+		catch (std::exception& e)
+        {
+			std::string err;
+			recursive_copy(old_path, new_path, err);
+			if (!err.empty())
+			{
+				return true;
+			}
+			m_save_path = save_path;
+			recursive_remove(old_path);
+        }
 	}
 
 #ifndef NDEBUG
@@ -1688,6 +1761,12 @@ namespace libtorrent
 
 		try
 		{
+			if (m_info->num_pieces() == 0)
+			{
+				m_state = state_create_files;
+				return std::make_pair(false, 1.f);
+			}
+
 			// initialization for the full check
 			if (m_hash_to_piece.empty())
 			{
@@ -1915,7 +1994,7 @@ namespace libtorrent
 		{
 			// find the file that failed, and skip all the blocks in that file
 			size_type file_offset = 0;
-			size_type current_offset = m_current_slot * m_info->piece_length();
+			size_type current_offset = size_type(m_current_slot) * m_info->piece_length();
 			for (torrent_info::file_iterator i = m_info->begin_files(true);
 					i != m_info->end_files(true); ++i)
 			{
@@ -2182,7 +2261,8 @@ namespace libtorrent
 
 		if (m_unallocated_slots.empty() && m_state == state_finished)
 		{
-			TORRENT_ASSERT(m_storage_mode != storage_mode_compact);
+			TORRENT_ASSERT(m_storage_mode != storage_mode_compact
+				|| m_info->num_pieces() == 0);
 		}
 		
 		if (m_storage_mode != storage_mode_compact)
