@@ -71,7 +71,7 @@ upnp::upnp(io_service& ios, connection_queue& cc
 	, m_io_service(ios)
 	, m_strand(ios)
 	, m_socket(ios, udp::endpoint(address_v4::from_string("239.255.255.250"), 1900)
-		, m_strand.wrap(bind(&upnp::on_reply, self(), _1, _2, _3)), false)
+		, bind(&upnp::on_reply, self(), _1, _2, _3), false)
 	, m_broadcast_timer(ios)
 	, m_refresh_timer(ios)
 	, m_disabled(false)
@@ -119,8 +119,8 @@ void upnp::discover_device() try
 
 	++m_retry_count;
 	m_broadcast_timer.expires_from_now(milliseconds(250 * m_retry_count));
-	m_broadcast_timer.async_wait(m_strand.wrap(bind(&upnp::resend_request
-		, self(), _1)));
+	m_broadcast_timer.async_wait(bind(&upnp::resend_request
+		, self(), _1));
 
 #ifdef TORRENT_UPNP_LOGGING
 	m_log << time_now_string()
@@ -209,9 +209,10 @@ try
 				m_log << time_now_string()
 					<< " ==> connecting to " << d.url << std::endl;
 #endif
+				if (d.upnp_connection) d.upnp_connection->close();
 				d.upnp_connection.reset(new http_connection(m_io_service
-					, m_cc, m_strand.wrap(bind(&upnp::on_upnp_xml, self(), _1, _2
-					, boost::ref(d)))));
+					, m_cc, bind(&upnp::on_upnp_xml, self(), _1, _2
+					, boost::ref(d), _5)));
 				d.upnp_connection->get(d.url);
 			}
 			catch (std::exception& e)
@@ -358,8 +359,20 @@ try
 		std::string protocol;
 		std::string auth;
 		// we don't have this device in our list. Add it
-		boost::tie(protocol, auth, d.hostname, d.port, d.path)
-			= parse_url_components(d.url);
+		try
+		{
+			boost::tie(protocol, auth, d.hostname, d.port, d.path)
+				= parse_url_components(d.url);
+		}
+		catch (std::exception& e)
+		{
+#ifdef TORRENT_UPNP_LOGGING
+			m_log << time_now_string()
+				<< " <== (" << from << ") invalid url: '" << d.url
+				<< "'. Ignoring device" << std::endl;
+#endif
+			return;
+		}
 
 		// ignore the auth here. It will be re-parsed
 		// by the http connection later
@@ -444,9 +457,10 @@ try
 					m_log << time_now_string()
 						<< " ==> connecting to " << d.url << std::endl;
 #endif
+					if (d.upnp_connection) d.upnp_connection->close();
 					d.upnp_connection.reset(new http_connection(m_io_service
-						, m_cc, m_strand.wrap(bind(&upnp::on_upnp_xml, self(), _1, _2
-						, boost::ref(d)))));
+						, m_cc, bind(&upnp::on_upnp_xml, self(), _1, _2
+						, boost::ref(d), _5)));
 					d.upnp_connection->get(d.url);
 				}
 				catch (std::exception& e)
@@ -552,9 +566,10 @@ void upnp::map_port(rootdevice& d, int i)
 		m_log << time_now_string()
 			<< " ==> connecting to " << d.hostname << std::endl;
 #endif
+	if (d.upnp_connection) d.upnp_connection->close();
 	d.upnp_connection.reset(new http_connection(m_io_service
-		, m_cc, m_strand.wrap(bind(&upnp::on_upnp_map_response, self(), _1, _2
-		, boost::ref(d), i)), true
+		, m_cc, bind(&upnp::on_upnp_map_response, self(), _1, _2
+		, boost::ref(d), i, _5), true
 		, bind(&upnp::create_port_mapping, self(), _1, boost::ref(d), i)));
 
 	d.upnp_connection->start(d.hostname, boost::lexical_cast<std::string>(d.port)
@@ -609,9 +624,11 @@ void upnp::unmap_port(rootdevice& d, int i)
 		m_log << time_now_string()
 			<< " ==> connecting to " << d.hostname << std::endl;
 #endif
+
+	if (d.upnp_connection) d.upnp_connection->close();
 	d.upnp_connection.reset(new http_connection(m_io_service
-		, m_cc, m_strand.wrap(bind(&upnp::on_upnp_unmap_response, self(), _1, _2
-		, boost::ref(d), i)), true
+		, m_cc, bind(&upnp::on_upnp_unmap_response, self(), _1, _2
+		, boost::ref(d), i, _5), true
 		, bind(&upnp::delete_port_mapping, self(), boost::ref(d), i)));
 	d.upnp_connection->start(d.hostname, boost::lexical_cast<std::string>(d.port)
 		, seconds(10));
@@ -675,10 +692,11 @@ namespace
 }
 
 void upnp::on_upnp_xml(asio::error_code const& e
-	, libtorrent::http_parser const& p, rootdevice& d) try
+	, libtorrent::http_parser const& p, rootdevice& d
+	, http_connection& c) try
 {
 	TORRENT_ASSERT(d.magic == 1337);
-	if (d.upnp_connection)
+	if (d.upnp_connection && d.upnp_connection.get() == &c)
 	{
 		d.upnp_connection->close();
 		d.upnp_connection.reset();
@@ -823,10 +841,11 @@ namespace
 }
 
 void upnp::on_upnp_map_response(asio::error_code const& e
-	, libtorrent::http_parser const& p, rootdevice& d, int mapping) try
+	, libtorrent::http_parser const& p, rootdevice& d, int mapping
+	, http_connection& c) try
 {
 	TORRENT_ASSERT(d.magic == 1337);
-	if (d.upnp_connection)
+	if (d.upnp_connection && d.upnp_connection.get() == &c)
 	{
 		d.upnp_connection->close();
 		d.upnp_connection.reset();
@@ -945,7 +964,7 @@ void upnp::on_upnp_map_response(asio::error_code const& e
 				|| next_expire > d.mapping[mapping].expires)
 			{
 				m_refresh_timer.expires_at(d.mapping[mapping].expires);
-				m_refresh_timer.async_wait(m_strand.wrap(bind(&upnp::on_expire, self(), _1)));
+				m_refresh_timer.async_wait(bind(&upnp::on_expire, self(), _1));
 			}
 		}
 		else
@@ -969,10 +988,11 @@ catch (std::exception&)
 };
 
 void upnp::on_upnp_unmap_response(asio::error_code const& e
-	, libtorrent::http_parser const& p, rootdevice& d, int mapping) try
+	, libtorrent::http_parser const& p, rootdevice& d, int mapping
+	, http_connection& c) try
 {
 	TORRENT_ASSERT(d.magic == 1337);
-	if (d.upnp_connection)
+	if (d.upnp_connection && d.upnp_connection.get() == &c)
 	{
 		d.upnp_connection->close();
 		d.upnp_connection.reset();
@@ -1054,7 +1074,7 @@ void upnp::on_expire(asio::error_code const& e) try
 	if (next_expire != max_time())
 	{
 		m_refresh_timer.expires_at(next_expire);
-		m_refresh_timer.async_wait(m_strand.wrap(bind(&upnp::on_expire, self(), _1)));
+		m_refresh_timer.async_wait(bind(&upnp::on_expire, self(), _1));
 	}
 }
 catch (std::exception&)
