@@ -32,7 +32,7 @@ import os
 import xdg.BaseDirectory
 
 PROGRAM_NAME = "Deluge"
-PROGRAM_VERSION = "0.5.8.9"
+PROGRAM_VERSION = "0.5.9.3"
 
 CLIENT_CODE = "DE"
 CLIENT_VERSION = "".join(PROGRAM_VERSION.split('.'))+"0"*(4 - len(PROGRAM_VERSION.split('.')))
@@ -185,16 +185,17 @@ def fetch_url(url):
             
     return None
 
-def exec_command(executable=None, *parameters):
+def exec_command(executable, *parameters):
     import os
+    command = [executable] 
+    command.extend(parameters) 
     if windows_check():
         try:
-            os.startfile(parameters)
+            from subprocess import Popen 
+            Popen(command)
         except:
             pass
     else:
-        command = [executable]
-        command.extend(parameters)
         try:
             os.WEXITSTATUS(os.system(command[0] + " \"%s\"" %command[1]))
         except OSError:
