@@ -37,8 +37,8 @@ import pygtk
 pygtk.require('2.0')
 import gtk
 
-import core
 import common
+import core
 import dialogs
 import dgtk
 import ipc_manager
@@ -46,12 +46,16 @@ import plugins
 import tab_details
 
 class DelugeGTK:
-    def __init__(self):
+    def __init__(self, config_dir=None):
+        if config_dir:
+            common.CONFIG_DIR = config_dir
+            base_change = True
         self.ipc_manager = ipc_manager.Manager(self)
         #Start the Deluge Manager:
         self.manager = core.Manager(common.CLIENT_CODE, common.CLIENT_VERSION, 
             '%s %s' % (common.PROGRAM_NAME, common.PROGRAM_VERSION), 
-            common.CONFIG_DIR)
+            common.CONFIG_DIR, base_change=base_change)
+
         self.plugins = plugins.PluginManager(self.manager, self)
         self.plugins.add_plugin_dir(common.PLUGIN_DIR)
         if os.path.isdir(os.path.join(common.CONFIG_DIR , 'plugins')):
@@ -267,6 +271,7 @@ and MOZILLA_FIVE_HOME environment variables to /usr/lib/firefox"))
             "pause_all": self.pause_all_clicked,
             "resume_all": self.resume_all_clicked,
             "add_torrent": self.add_torrent_clicked,
+            "add_torrent_url": self.add_torrent_url_clicked,
             "show_hide_window_toggled": self.show_hide_window_toggled
         })
         
@@ -739,9 +744,9 @@ window, please enter your password"))
                 torrent_path = self.manager.get_torrent_path(uid)
                 if not common.windows_check():
                     file_manager = "xdg-open"
-                    common.exec_command(file_manager, torrent_path)
                 else:
-                    common.exec_command(executable=None, parameters=torrent_path)
+                    file_manager = "explorer.exe"
+                common.exec_command(file_manager, torrent_path)
                     
         except KeyError:
             pass
@@ -772,6 +777,7 @@ window, please enter your password"))
                      trackers_changed = 0
                 self.manager.pickle_state()
                 uploaded_memory = self.manager.unique_IDs[uid].uploaded_memory
+                uploaded_memory += self.manager.get_core_torrent_state(uid, False)['total_payload_upload']
                 priorities = self.manager.get_priorities(uid)
                 save_info = [path, save_dir, order, trackerslist, \
                                 uploaded_memory, priorities, trackers_changed, \
@@ -1009,7 +1015,6 @@ window, please enter your password"))
     def start(self, cmd_line_torrents=None):
         if cmd_line_torrents is None:
             cmd_line_torrents = []
-        
         if not(self.config.get("start_in_tray") and \
                self.config.get("enable_system_tray") and 
                self.has_tray) and not self.window.get_property("visible"):
