@@ -41,9 +41,10 @@ import xdg
 
 if common.windows_check():
     DEFAULT_PREFS = {
-    "enabled_plugins" : "Anonymizing Browser:Torrent Files:Torrent Peers:Torrent Peers:Torrent Notification",
+    "enabled_plugins" : "Torrent Files:Torrent Peers:Torrent Peers:Torrent Notification",
     "file_manager" : common.FileManager.xdg,
     "open_folder_stock" : True,
+    "send_redund" : False,
     "autoload" : False,
     "open_folder_location": "",
     "send_info" : True,
@@ -58,7 +59,7 @@ if common.windows_check():
     "clear_max_ratio_torrents" : False,
     "clear_max_time_torrents" : False,
     "default_autoload_path" : os.path.expanduser("~"),
-    "default_torrent_path" : os.path.join(os.path.expanduser("~"), 'deluge', 'torrentfiles'),
+    "default_torrent_path" : os.path.join(common.CONFIG_DIR, 'torrentfiles'),
     "default_download_path" : os.path.expanduser("~"),
     "open_torrent_dialog_path" : os.path.expanduser("~"),
     "choose_directory_dialog_path": os.path.expanduser("~"),
@@ -162,6 +163,7 @@ else:
     "enabled_plugins" : "Torrent Files:Torrent Peers:Torrent Notification",
     "file_manager" : common.FileManager.xdg,
     "open_folder_stock" : True,
+    "send_redund" : False,
     "autoload" : False,
     "open_folder_location": "",
     "send_info" : True,
@@ -177,7 +179,7 @@ else:
     "clear_max_ratio_torrents" : False,
     "clear_max_time_torrents" : False,
     "default_autoload_path" : os.path.expanduser("~"),
-    "default_torrent_path" : os.path.join(xdg.BaseDirectory.save_config_path('deluge'), 'torrentfiles'),
+    "default_torrent_path" : os.path.join(common.CONFIG_DIR, 'torrentfiles'),
     "default_download_path" : os.path.expanduser("~"),
     "open_torrent_dialog_path" : os.path.expanduser("~"),
     "choose_directory_dialog_path": os.path.expanduser("~"),
@@ -305,7 +307,7 @@ class Preferences:
         if key not in self.mapping or self.mapping[key]!=value:
             self.mapping[key] = value 
             for hook in self.change_hooks: 
- 	        if (hook[0]==key): hook[1]() 
+                if (hook[0]==key): hook[1]() 
 
     def __delitem__(self, key):
         del self.mapping[key]
